using System;
using System.Data;
using System.Data.SqlClient;
using System.Data.SqlTypes;
using Microsoft.SqlServer.Server;
using System.Collections;
using System.Collections.Generic;

public partial class UserDefinedFunctions
{
    public struct BBox
    {
        public SqlString _PlaceName;
        public SqlString _State;
        public SqlDouble _Dist;
        public BBox(SqlString PlaceName, SqlString State, SqlDouble Dist)
        {
            _PlaceName = PlaceName;
            _State = State;
            _Dist = Dist;
        }
    }

    [Microsoft.SqlServer.Server.SqlFunction(IsDeterministic = true,
      DataAccess = DataAccessKind.Read, FillRowMethodName = "FillRow",
      TableDefinition = "PlaceName NVARCHAR(100), State NCHAR(2), Dist FLOAT")]
    public static IEnumerable CLRNeighborsWrong(SqlDouble LatIn, SqlDouble LonIn,
                                                SqlDouble MaxDistIn, SqlString Unit)
    {
        double Lat = LatIn.Value, Lon = LonIn.Value, MaxDist = MaxDistIn.Value;
        double LatMaxR, LatMinR, LonMaxR, LonMinR, Dummy;
        bool Miles = (Unit.Value.Equals("mi"));

        // Convert degrees to radians
        double LatR = Lat * D2R;
        double LonR = Lon * D2R;
        double MaxDistR = StoredProcedures.DistToRadians(MaxDist, Miles);
        const double HALFPI = Math.PI / 2;

        // Calculate minimum and maximum longitude and latitude
        StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, 0,   // North
                                     out LatMaxR, out Dummy);
        StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, Math.PI, // South
                                     out LatMinR, out Dummy);
        StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, HALFPI,  // West
                                     out Dummy, out LonMinR);
        StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, -HALFPI, // East
                                     out Dummy, out LonMaxR);

        // Convert back to degrees; add or subtract a tiny bit to fix rounding errors
        double LatMax = LatMaxR * R2D + 1e-10;
        double LatMin = LatMinR * R2D - 1e-10;
        double LonMin = LonMinR * R2D - 1e-10;
        double LonMax = LonMaxR * R2D + 1e-10;

        List<BBox> _BBdata = new List<BBox>();

        using (SqlConnection conn =
            new SqlConnection("context connection = true"))
        {
            SqlCommand comm =
                new SqlCommand("SELECT PlaceName, State, Lat, Lon "
                             + "FROM dbo.Place "
                             + "WHERE Lat BETWEEN @LatMin AND @LatMax "
                             + "AND Lon BETWEEN @LonMin AND @LonMax", conn);
            comm.Parameters.Add("@LatMin", SqlDbType.Float);
            comm.Parameters[0].Value = LatMin;
            comm.Parameters.Add("@LatMax", SqlDbType.Float);
            comm.Parameters[1].Value = LatMax;
            comm.Parameters.Add("@LonMin", SqlDbType.Float);
            comm.Parameters[2].Value = LonMin;
            comm.Parameters.Add("@LonMax", SqlDbType.Float);
            comm.Parameters[3].Value = LonMax;

            conn.Open();
            SqlDataReader reader = comm.ExecuteReader();

            while (reader.Read())
            {
                double Lat2 = reader.GetDouble(2);
                double Lon2 = reader.GetDouble(3);
                double Dist = SpatialDist(Lat, Lon, Lat2, Lon2, Miles);
                if (Dist <= MaxDist)
                {
                    SqlString PlaceName = reader.GetSqlString(0);
                    SqlString State = reader.GetSqlString(1);
                    BBox BBnew = new BBox(PlaceName,
                                            State,
                                            (SqlDouble)Dist);
                    _BBdata.Add(BBnew);
                }
            }
        }

        return (IEnumerable)_BBdata;
    }

    [Microsoft.SqlServer.Server.SqlFunction(IsDeterministic = true,
      DataAccess = DataAccessKind.Read, FillRowMethodName = "FillRow",
      TableDefinition = "PlaceName NVARCHAR(100), State NCHAR(2), Dist FLOAT")]
    public static IEnumerable CLRNeighborsFix1(SqlDouble LatIn, SqlDouble LonIn,
                                               SqlDouble MaxDistIn, SqlString Unit)
    {
        double Lat = LatIn.Value, Lon = LonIn.Value, MaxDist = MaxDistIn.Value;
        double LatMaxR, LatMinR, LonMaxR, LonMinR, Dummy;
        bool Miles = (Unit.Value.Equals("mi"));

        // Convert degrees to radians
        double LatR = Lat * D2R;
        double LonR = Lon * D2R;
        double MaxDistR = StoredProcedures.DistToRadians(MaxDist, Miles);
        const double HALFPI = Math.PI / 2;

        // Calculate minimum and maximum longitude and latitude
        StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, 0,   // North
                                     out LatMaxR, out Dummy);
        StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, Math.PI, // South
                                     out LatMinR, out Dummy);
        // Go east/west from north or south, depending on hemisphere
        double LatHelpR = (LatR > 0 ? LatMaxR : LatMinR);
        StoredProcedures.MoveRadians(LatHelpR, LonR, MaxDistR, HALFPI,  // West
                                     out Dummy, out LonMinR);
        StoredProcedures.MoveRadians(LatHelpR, LonR, MaxDistR, -HALFPI, // East
                                     out Dummy, out LonMaxR);

        // Convert back to degrees; add or subtract a tiny bit to latitude to fix rounding errors
        // Don't worry about rounding for longitude, since we're already overshooting
        double LatMax = LatMaxR * R2D + 1e-10;
        double LatMin = LatMinR * R2D - 1e-10;
        double LonMin = LonMinR * R2D;
        double LonMax = LonMaxR * R2D;

        List<BBox> _BBdata = new List<BBox>();

        using (SqlConnection conn =
            new SqlConnection("context connection = true"))
        {
            SqlCommand comm =
                new SqlCommand("SELECT PlaceName, State, Lat, Lon "
                             + "FROM dbo.Place "
                             + "WHERE Lat BETWEEN @LatMin AND @LatMax "
                             + "AND Lon BETWEEN @LonMin AND @LonMax", conn);
            comm.Parameters.Add("@LatMin", SqlDbType.Float);
            comm.Parameters[0].Value = LatMin;
            comm.Parameters.Add("@LatMax", SqlDbType.Float);
            comm.Parameters[1].Value = LatMax;
            comm.Parameters.Add("@LonMin", SqlDbType.Float);
            comm.Parameters[2].Value = LonMin;
            comm.Parameters.Add("@LonMax", SqlDbType.Float);
            comm.Parameters[3].Value = LonMax;

            conn.Open();
            SqlDataReader reader = comm.ExecuteReader();

            while (reader.Read())
            {
                double Lat2 = reader.GetDouble(2);
                double Lon2 = reader.GetDouble(3);
                double Dist = SpatialDist(Lat, Lon, Lat2, Lon2, Miles);
                if (Dist <= MaxDist)
                {
                    SqlString PlaceName = reader.GetSqlString(0);
                    SqlString State = reader.GetSqlString(1);
                    BBox BBnew = new BBox(PlaceName,
                                            State,
                                            (SqlDouble)Dist);
                    _BBdata.Add(BBnew);
                }
            }
        }

        return (IEnumerable)_BBdata;
    }

    [Microsoft.SqlServer.Server.SqlFunction(IsDeterministic = true,
      DataAccess = DataAccessKind.Read, FillRowMethodName = "FillRow",
      TableDefinition = "PlaceName NVARCHAR(100), State NCHAR(2), Dist FLOAT")]
    public static IEnumerable CLRNeighborsFix2(SqlDouble LatIn, SqlDouble LonIn,
                                               SqlDouble MaxDistIn, SqlString Unit)
    {
        double Lat = LatIn.Value, Lon = LonIn.Value, MaxDist = MaxDistIn.Value;
        double LatMaxR, LatMinR, LonMaxR, LonMinR, Dummy;
        bool Miles = (Unit.Value.Equals("mi"));

        // Convert degrees to radians
        double LatR = Lat * D2R;
        double LonR = Lon * D2R;
        double MaxDistR = StoredProcedures.DistToRadians(MaxDist, Miles);

        // Calculate minimum and maximum longitude and latitude
        StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, 0,   // North
                                     out LatMaxR, out Dummy);
        StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, Math.PI, // South
                                     out LatMinR, out Dummy);
        // Calculate direction to get longitude of west boundary
        double DirHelpR = Math.Acos(Math.Tan(MaxDistR) * Math.Tan(LatR));
        StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, DirHelpR,
                                     out Dummy, out LonMinR);
        // Direction for east boundary is -direction for west boundary
        StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, -DirHelpR,
                                     out Dummy, out LonMaxR);

        // Convert back to degrees; add or subtract a tiny bit to fix rounding errors
        double LatMax = LatMaxR * R2D + 1e-10;
        double LatMin = LatMinR * R2D - 1e-10;
        double LonMin = LonMinR * R2D - 1e-10;
        double LonMax = LonMaxR * R2D + 1e-10;

        List<BBox> _BBdata = new List<BBox>();

        using (SqlConnection conn =
            new SqlConnection("context connection = true"))
        {
            SqlCommand comm =
                new SqlCommand("SELECT PlaceName, State, Lat, Lon "
                             + "FROM dbo.Place "
                             + "WHERE Lat BETWEEN @LatMin AND @LatMax "
                             + "AND Lon BETWEEN @LonMin AND @LonMax", conn);
            comm.Parameters.Add("@LatMin", SqlDbType.Float);
            comm.Parameters[0].Value = LatMin;
            comm.Parameters.Add("@LatMax", SqlDbType.Float);
            comm.Parameters[1].Value = LatMax;
            comm.Parameters.Add("@LonMin", SqlDbType.Float);
            comm.Parameters[2].Value = LonMin;
            comm.Parameters.Add("@LonMax", SqlDbType.Float);
            comm.Parameters[3].Value = LonMax;

            conn.Open();
            SqlDataReader reader = comm.ExecuteReader();

            while (reader.Read())
            {
                double Lat2 = reader.GetDouble(2);
                double Lon2 = reader.GetDouble(3);
                double Dist = SpatialDist(Lat, Lon, Lat2, Lon2, Miles);
                if (Dist <= MaxDist)
                {
                    SqlString PlaceName = reader.GetSqlString(0);
                    SqlString State = reader.GetSqlString(1);
                    BBox BBnew = new BBox(PlaceName,
                                            State,
                                            (SqlDouble)Dist);
                    _BBdata.Add(BBnew);
                }
            }
        }

        return (IEnumerable)_BBdata;
    }

    [Microsoft.SqlServer.Server.SqlFunction(IsDeterministic = true,
      DataAccess = DataAccessKind.Read, FillRowMethodName = "FillRow",
      TableDefinition = "PlaceName NVARCHAR(100), State NCHAR(2), Dist FLOAT")]
    public static IEnumerable CLRDynamicBBWrong(SqlDouble LatIn, SqlDouble LonIn,
                                                SqlString Unit)
    {
        double Lat = LatIn.Value, Lon = LonIn.Value, MaxDist = 100000;
        double LatMaxR, LatMinR, LonMaxR, LonMinR, Dummy;
        bool Miles = (Unit.Value.Equals("mi"));
        double Lat2, Lon2, Dist;
        BBox[] BBdata = new BBox[1];

        // Find MaxDist to use; try Tries locations east and west,
        //                      but stop at Threshold.
        using (SqlConnection conn = new SqlConnection("context connection = true"))
        {
            conn.Open();

            // Sample some places
            SqlCommand comm1 = new SqlCommand("SELECT Lat, Lon FROM "
                          + "(SELECT TOP(26) Lat, Lon "
                          + "FROM dbo.Place WHERE Lon > @Lon "
                          + "ORDER BY Lon ASC) AS East "
                          + "UNION ALL SELECT Lat, Lon FROM "
                          + "(SELECT TOP(26) Lat, Lon "
                          + "FROM dbo.Place WHERE Lon < @Lon "
                          + "ORDER BY Lon DESC) AS West", conn);
            comm1.Parameters.Add("@Lon", SqlDbType.Float);
            comm1.Parameters[0].Value = Lon;

            using (SqlDataReader reader = comm1.ExecuteReader())
            {
                // Bail out when below threshold
                while ((MaxDist > 8.5) && (reader.Read()))
                {
                    Lat2 = reader.GetDouble(0);
                    Lon2 = reader.GetDouble(1);
                    Dist = SpatialDist(Lat, Lon, Lat2, Lon2, Miles);
                    if (Dist <= MaxDist)
                        MaxDist = Dist;
                }
            }

            // Add tiny bit to MinDist to fence off rounding errors
            MaxDist += 0.001;

            // Convert degrees to radians
            double LatR = Lat * D2R;
            double LonR = Lon * D2R;
            double MaxDistR = StoredProcedures.DistToRadians(MaxDist, Miles);
            const double HALFPI = Math.PI / 2;

            // Calculate minimum and maximum longitude and latitude
            StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, 0,   // North
                                         out LatMaxR, out Dummy);
            StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, Math.PI, // South
                                         out LatMinR, out Dummy);
            StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, HALFPI,  // West
                                         out Dummy, out LonMinR);
            StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, -HALFPI, // East
                                         out Dummy, out LonMaxR);

            // Convert back to degrees; don't worry about rounding here as we already added 0.001 to MaxDist
            double LatMax = LatMaxR * R2D;
            double LatMin = LatMinR * R2D;
            double LonMin = LonMinR * R2D;
            double LonMax = LonMaxR * R2D;

            // Fetch rows within the dynamic bounding box
            SqlCommand comm2 = new SqlCommand("SELECT PlaceName, State, Lat, Lon "
                                   + "FROM dbo.Place "
                                   + "WHERE Lat BETWEEN @LatMin AND @LatMax "
                                   + "AND Lon BETWEEN @LonMin AND @LonMax", conn);
            comm2.Parameters.Add("@LatMin", SqlDbType.Float);
            comm2.Parameters[0].Value = LatMin;
            comm2.Parameters.Add("@LatMax", SqlDbType.Float);
            comm2.Parameters[1].Value = LatMax;
            comm2.Parameters.Add("@LonMin", SqlDbType.Float);
            comm2.Parameters[2].Value = LonMin;
            comm2.Parameters.Add("@LonMax", SqlDbType.Float);
            comm2.Parameters[3].Value = LonMax;

            using (SqlDataReader reader = comm2.ExecuteReader())
            {
                // Find place with lowest non-zero distance
                double MinDist = MaxDist;

                while (reader.Read())
                {
                    Lat2 = reader.GetDouble(2);
                    Lon2 = reader.GetDouble(3);
                    if ((Lat2 != Lat) || (Lon2 != Lon))
                    {
                        Dist = SpatialDist(Lat, Lon, Lat2, Lon2, Miles);
                        if (Dist < MinDist)
                        {
                            MinDist = Dist;
                            BBdata[0]._PlaceName = reader.GetSqlString(0);
                            BBdata[0]._State = reader.GetSqlString(1);
                        }
                    }
                }
                BBdata[0]._Dist = MinDist;
            }
        }

        return (IEnumerable)BBdata;
    }

    [Microsoft.SqlServer.Server.SqlFunction(IsDeterministic = true,
      DataAccess = DataAccessKind.Read, FillRowMethodName = "FillRow",
      TableDefinition = "PlaceName NVARCHAR(100), State NCHAR(2), Dist FLOAT")]
    public static IEnumerable CLRDynamicBBFix1(SqlDouble LatIn, SqlDouble LonIn,
                                               SqlString Unit)
    {
        double Lat = LatIn.Value, Lon = LonIn.Value, MaxDist = 100000;
        double LatMaxR, LatMinR, LonMaxR, LonMinR, Dummy;
        bool Miles = (Unit.Value.Equals("mi"));
        double Lat2, Lon2, Dist;
        BBox[] BBdata = new BBox[1];

        // Find MaxDist to use; try Tries locations east and west,
        //                      but stop at Threshold.
        using (SqlConnection conn = new SqlConnection("context connection = true"))
        {
            conn.Open();

            // Sample some places
            SqlCommand comm1 = new SqlCommand("SELECT Lat, Lon FROM "
                          + "(SELECT TOP(26) Lat, Lon "
                          + "FROM dbo.Place WHERE Lon > @Lon "
                          + "ORDER BY Lon ASC) AS East "
                          + "UNION ALL SELECT Lat, Lon FROM "
                          + "(SELECT TOP(26) Lat, Lon "
                          + "FROM dbo.Place WHERE Lon < @Lon "
                          + "ORDER BY Lon DESC) AS West", conn);
            comm1.Parameters.Add("@Lon", SqlDbType.Float);
            comm1.Parameters[0].Value = Lon;

            using (SqlDataReader reader = comm1.ExecuteReader())
            {
                // Bail out when below threshold
                while ((MaxDist > 8.5) && (reader.Read()))
                {
                    Lat2 = reader.GetDouble(0);
                    Lon2 = reader.GetDouble(1);
                    Dist = SpatialDist(Lat, Lon, Lat2, Lon2, Miles);
                    if (Dist <= MaxDist)
                        MaxDist = Dist;
                }
            }

            // Add tiny bit to MinDist to fence off rounding errors
            MaxDist += 0.001;

            // Convert degrees to radians
            double LatR = Lat * D2R;
            double LonR = Lon * D2R;
            double MaxDistR = StoredProcedures.DistToRadians(MaxDist, Miles);
            const double HALFPI = Math.PI / 2;

            // Calculate minimum and maximum longitude and latitude
            StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, 0,   // North
                                         out LatMaxR, out Dummy);
            StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, Math.PI, // South
                                         out LatMinR, out Dummy);
            // Go east/west from north or south, depending on hemisphere
            double LatHelpR = (LatR > 0 ? LatMaxR : LatMinR);
            StoredProcedures.MoveRadians(LatHelpR, LonR, MaxDistR, HALFPI,  // West
                                         out Dummy, out LonMinR);
            StoredProcedures.MoveRadians(LatHelpR, LonR, MaxDistR, -HALFPI, // East
                                         out Dummy, out LonMaxR);

            // Convert back to degrees; don't worry about rounding here as we already added 0.001 to MaxDist
            double LatMax = LatMaxR * R2D;
            double LatMin = LatMinR * R2D;
            double LonMin = LonMinR * R2D;
            double LonMax = LonMaxR * R2D;

            // Fetch rows within the dynamic bounding box
            SqlCommand comm2 = new SqlCommand("SELECT PlaceName, State, Lat, Lon "
                                   + "FROM dbo.Place "
                                   + "WHERE Lat BETWEEN @LatMin AND @LatMax "
                                   + "AND Lon BETWEEN @LonMin AND @LonMax", conn);
            comm2.Parameters.Add("@LatMin", SqlDbType.Float);
            comm2.Parameters[0].Value = LatMin;
            comm2.Parameters.Add("@LatMax", SqlDbType.Float);
            comm2.Parameters[1].Value = LatMax;
            comm2.Parameters.Add("@LonMin", SqlDbType.Float);
            comm2.Parameters[2].Value = LonMin;
            comm2.Parameters.Add("@LonMax", SqlDbType.Float);
            comm2.Parameters[3].Value = LonMax;

            using (SqlDataReader reader = comm2.ExecuteReader())
            {
                // Find place with lowest non-zero distance
                double MinDist = MaxDist;

                while (reader.Read())
                {
                    Lat2 = reader.GetDouble(2);
                    Lon2 = reader.GetDouble(3);
                    if ((Lat2 != Lat) || (Lon2 != Lon))
                    {
                        Dist = SpatialDist(Lat, Lon, Lat2, Lon2, Miles);
                        if (Dist < MinDist)
                        {
                            MinDist = Dist;
                            BBdata[0]._PlaceName = reader.GetSqlString(0);
                            BBdata[0]._State = reader.GetSqlString(1);
                        }
                    }
                }
                BBdata[0]._Dist = MinDist;
            }
        }

        return (IEnumerable)BBdata;
    }

    [Microsoft.SqlServer.Server.SqlFunction(IsDeterministic = true,
      DataAccess = DataAccessKind.Read, FillRowMethodName = "FillRow",
      TableDefinition = "PlaceName NVARCHAR(100), State NCHAR(2), Dist FLOAT")]
    public static IEnumerable CLRDynamicBBFix2(SqlDouble LatIn, SqlDouble LonIn,
                                               SqlString Unit)
    {
        double Lat = LatIn.Value, Lon = LonIn.Value, MaxDist = 100000;
        double LatMaxR, LatMinR, LonMaxR, LonMinR, Dummy;
        bool Miles = (Unit.Value.Equals("mi"));
        double Lat2, Lon2, Dist;
        BBox[] BBdata = new BBox[1];

        // Find MaxDist to use; try Tries locations east and west,
        //                      but stop at Threshold.
        using (SqlConnection conn = new SqlConnection("context connection = true"))
        {
            conn.Open();

            // Sample some places
            SqlCommand comm1 = new SqlCommand("SELECT Lat, Lon FROM "
                          + "(SELECT TOP(26) Lat, Lon "
                          + "FROM dbo.Place WHERE Lon > @Lon "
                          + "ORDER BY Lon ASC) AS East "
                          + "UNION ALL SELECT Lat, Lon FROM "
                          + "(SELECT TOP(26) Lat, Lon "
                          + "FROM dbo.Place WHERE Lon < @Lon "
                          + "ORDER BY Lon DESC) AS West", conn);
            comm1.Parameters.Add("@Lon", SqlDbType.Float);
            comm1.Parameters[0].Value = Lon;

            using (SqlDataReader reader = comm1.ExecuteReader())
            {
                // Bail out when below threshold
                while ((MaxDist > 8.5) && (reader.Read()))
                {
                    Lat2 = reader.GetDouble(0);
                    Lon2 = reader.GetDouble(1);
                    Dist = SpatialDist(Lat, Lon, Lat2, Lon2, Miles);
                    if (Dist <= MaxDist)
                        MaxDist = Dist;
                }
            }

            // Add tiny bit to MinDist to fence off rounding errors
            MaxDist += 0.001;

            // Convert degrees to radians
            double LatR = Lat * D2R;
            double LonR = Lon * D2R;
            double MaxDistR = StoredProcedures.DistToRadians(MaxDist, Miles);

            // Calculate minimum and maximum longitude and latitude
            StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, 0,   // North
                                         out LatMaxR, out Dummy);
            StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, Math.PI, // South
                                         out LatMinR, out Dummy);
            // Calculate direction to get longitude of west boundary
            double DirHelpR = Math.Acos(Math.Tan(MaxDistR) * Math.Tan(LatR));
            StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, DirHelpR,
                                         out Dummy, out LonMinR);
            // Direction for east boundary is -direction for west boundary
            StoredProcedures.MoveRadians(LatR, LonR, MaxDistR, -DirHelpR,
                                         out Dummy, out LonMaxR);

            // Convert back to degrees; don't worry about rounding here as we already added 0.001 to MaxDist
            double LatMax = LatMaxR * R2D;
            double LatMin = LatMinR * R2D;
            double LonMin = LonMinR * R2D;
            double LonMax = LonMaxR * R2D;

            // Fetch rows within the dynamic bounding box
            SqlCommand comm2 = new SqlCommand("SELECT PlaceName, State, Lat, Lon "
                                   + "FROM dbo.Place "
                                   + "WHERE Lat BETWEEN @LatMin AND @LatMax "
                                   + "AND Lon BETWEEN @LonMin AND @LonMax", conn);
            comm2.Parameters.Add("@LatMin", SqlDbType.Float);
            comm2.Parameters[0].Value = LatMin;
            comm2.Parameters.Add("@LatMax", SqlDbType.Float);
            comm2.Parameters[1].Value = LatMax;
            comm2.Parameters.Add("@LonMin", SqlDbType.Float);
            comm2.Parameters[2].Value = LonMin;
            comm2.Parameters.Add("@LonMax", SqlDbType.Float);
            comm2.Parameters[3].Value = LonMax;

            using (SqlDataReader reader = comm2.ExecuteReader())
            {
                // Find place with lowest non-zero distance
                double MinDist = MaxDist;

                while (reader.Read())
                {
                    Lat2 = reader.GetDouble(2);
                    Lon2 = reader.GetDouble(3);
                    if ((Lat2 != Lat) || (Lon2 != Lon))
                    {
                        Dist = SpatialDist(Lat, Lon, Lat2, Lon2, Miles);
                        if (Dist < MinDist)
                        {
                            MinDist = Dist;
                            BBdata[0]._PlaceName = reader.GetSqlString(0);
                            BBdata[0]._State = reader.GetSqlString(1);
                        }
                    }
                }
                BBdata[0]._Dist = MinDist;
            }
        }

        return (IEnumerable)BBdata;
    }

    public static void FillRow(Object obj, out SqlString PlaceName,
                               out SqlString State, out SqlDouble Dist)
    {
        BBox bb = (BBox)obj;
        PlaceName = bb._PlaceName;
        State = bb._State;
        Dist = bb._Dist;
    }
};