using System;
using System.Data;
using System.Data.SqlClient;
using System.Data.SqlTypes;
using Microsoft.SqlServer.Server;

public partial class UserDefinedFunctions
{
    // Constants for converting between degrees and radians
    const double D2R = Math.PI / 180;
    const double R2D = 180 / Math.PI;

    [Microsoft.SqlServer.Server.SqlFunction(IsDeterministic = true)]
    public static SqlDouble DistCLR
                 (SqlDouble Lat1, SqlDouble Lon1,
                  SqlDouble Lat2, SqlDouble Lon2, SqlString Unit)
    {
        // This is just a wrapper for the T-SQL interface.
        // Call the function that does the real work.
        double Dist = SpatialDist(Lat1.Value, Lon1.Value,
                                  Lat2.Value, Lon2.Value,
                                  (Unit.Value.Equals("mi")));
        return new SqlDouble(Dist);
    }


    public static double SpatialDist
                 (double Lat1, double Lon1,
                  double Lat2, double Lon2, bool Miles)
    {
        // Convert degrees to radians
        double Lat1R = Lat1 * D2R;
        double Lon1R = Lon1 * D2R;
        double Lat2R = Lat2 * D2R;
        double Lon2R = Lon2 * D2R;

        // Calculate distance
        double Help1 = Math.Sin((Lat1R - Lat2R) / 2);
        double Help2 = Help1 * Help1;
        double Help3 = Math.Sin((Lon1R - Lon2R) / 2);
        double Help4 = Help3 * Help3;
        double DistR = 2 * Math.Asin(Math.Sqrt(Help2 + (Math.Cos(Lat1R) * Math.Cos(Lat2R) * Help4)));

        // Convert from radians to kilometers or miles
        const double R2KM = 20001.6 / Math.PI;
        const double R2MI = 20001.6 / 1.609344 / Math.PI;
        double Dist = DistR * (Miles ? R2MI : R2KM);

        return Dist;
    }
};