using System;
using System.Data;
using System.Data.SqlClient;
using System.Data.SqlTypes;
using Microsoft.SqlServer.Server;

public partial class StoredProcedures
{
    // Constants for converting between degrees and radians
    const double D2R = Math.PI / 180;
    const double R2D = 180 / Math.PI;

    [Microsoft.SqlServer.Server.SqlProcedure]
    public static void MoveCLR
                 (SqlDouble Lat1, SqlDouble Lon1,
                  SqlDouble Dist, SqlDouble Dir,
                  out SqlDouble Lat2, out SqlDouble Lon2,
                  SqlString Unit)
    {
        // This is just a wrapper for the T-SQL interface.
        // Call the proc that does the real work.
        double Lat2d;
        double Lon2d;
        SpatialMove(Lat1.Value, Lon1.Value,
                    Dist.Value, Dir.Value,
                    (Unit.Value.Equals("mi")),
                    out Lat2d, out Lon2d);
        Lat2 = new SqlDouble(Lat2d);
        Lon2 = new SqlDouble(Lon2d);
    }

    public static void SpatialMove
                 (double Lat1, double Lon1,
                  double Dist, double Dir,
                  bool Miles,
                  out double Lat2, out double Lon2)
    {
        // Convert degrees to radians
        double Lat1R = Lat1 * D2R;
        double Lon1R = Lon1 * D2R;
        double DirR = Dir * D2R;
        double DistR = DistToRadians(Dist, Miles);

        // Delegate rest of the work
        double Lat2R;
        double Lon2R;
        MoveRadians(Lat1R, Lon1R, DistR, DirR, out Lat2R, out Lon2R);

        // Convert results back to degrees
        Lat2 = Lat2R * R2D;
        Lon2 = Lon2R * R2D;
    }
        
    public static void MoveRadians
                 (double Lat1R, double Lon1R,
                  double DistR, double DirR,
                  out double Lat2R, out double Lon2R)
    {
        const double TwoPI = 2 * Math.PI;
        
        // Precalculate terms we need more than once
        double SinLat1 = Math.Sin(Lat1R);
        double CosDist = Math.Cos(DistR);
        double Help = Math.Cos(Lat1R) * Math.Sin(DistR);

        // Calculate new latitude
        double SinLat2 = SinLat1 * CosDist + Help * Math.Cos(DirR);
        Lat2R = Math.Asin(SinLat2);

        // Calculate longitude difference
        double DLonR = Math.Atan2(Math.Sin(DirR) * Help,
                                  CosDist - SinLat1 * SinLat2);
        // Calculate new longitude - ensure result is between -PI and PI
        Lon2R = ((Lon1R - DLonR + Math.PI) % TwoPI) - Math.PI;
    }

    public static double DistToRadians(double Dist, bool Miles)
    {
        // Convert distance from km/mi to radians
        // Note: DistR = Distance in nautical miles * (pi / (180 * 60))
        //               One nautical mile is 1.852 kilometers, thus:
        //   or: DistR =(DistKM / 1.852) * pi / (180 * 60)
        //   or: DistR = DistKM * pi / (180 * 60 * 1.852)
        //   or: DistR = DistKM * pi / 20001.6
        // Since one mile is 1.609344 kilometers, the formula for miles is:
        //       DistR =(DistMI * 1.609344) * pi / 20001.6
        //   or: DistR = DistMI * pi / (20001.6 / 1.609344)
        const double KM2R = Math.PI / 20001.6;
        const double MI2R = Math.PI / (20001.6 / 1.609344);
        return Dist * (Miles ? MI2R : KM2R);
    }
};